# This file specifies the marker genes methods (and their parameters) which
# will be assessed.

suppressMessages({
  library(tibble)
  library(tidyr)
  library(dplyr)
  library(purrr)
})

# Read the overall config file.
config <- yaml::read_yaml(here::here("config.yaml"))

scran_pars <- expand_grid(
  method = "scran",
  test.type = c("t", "wilcox", "binom"),
  pval.type = c("any", "all", "some")
) %>%
  mutate(file_name = paste0(method, "_", test.type, "_", pval.type))

seurat_pars <- expand_grid(
  method = "seurat",
  test.use = c("wilcox", "bimod", "roc", "t", "negbinom", "poisson", "LR",
               "MAST")
) %>%
  mutate(file_name = paste0(method, "_", test.use))

lm_pars <- expand_grid(
  method = "lm",
  covariate = c("two_sample")
) %>%
  mutate(file_name = paste0(method, "_", covariate))

binom_pars <- expand_grid(
  method = "binom",
) %>%
  mutate(file_name = paste0(method))

edger_pars <- expand_grid(
  method = "edger",
  fit_method = c("ql", "glm"),
  covariate = c("cdr", "none")
) %>%
  mutate(file_name = paste0(method, "_", fit_method, "_", covariate))

limma_pars <- expand_grid(
  method = "limma",
  fit_method = c("voom", "trend")
) %>%
  mutate(file_name = paste0(method, "_", fit_method))

scanpy_pars <- expand_grid(
  method = "scanpy",
  test_use = c("t", "tover", "wilcoxon", "wilcoxontiecorrect"),
  rankby_abs = c("True", "False")
) %>%
  mutate(rankby_readable = if_else(
    rankby_abs == "True",
    "rankby_abs",
    "rankby_raw")
  ) %>%
  mutate(file_name = paste0(method, "_", test_use, "_", rankby_readable)) %>%
  select(-rankby_readable)

presto_pars <- expand_grid(
  method = "presto",
) %>%
  mutate(file_name = paste0(method))

rankcorr_pars <- expand_grid(
  method = "rankcorr",
  lambda = c(2, 5, 10, 20)
) %>%
  mutate(file_name = paste0(method, "_", lambda))

random_pars <- expand_grid(
  method = "random",
) %>%
  mutate(file_name = paste0(method))

difference_pars <- expand_grid(
  method = "difference",
  summary = c("log_fc")
) %>%
  mutate(file_name = paste0(method, "_", summary))

glmgampoi_pars <- expand_grid(
  method = "glmgampoi",
) %>%
  mutate(file_name = paste0(method))

method_pars_data <- list(
  scran_pars,
  seurat_pars,
  lm_pars,
  #binom_pars,
  edger_pars,
  limma_pars,
  scanpy_pars,
  presto_pars,
  rankcorr_pars,
  random_pars,
  difference_pars,
  glmgampoi_pars
)

method_pars <- flatten(map(method_pars_data, ~ transpose(.x, .names = .x$file_name)))
method_ids <- names(method_pars)

method_paths <- paste0(here::here(config$method_pars_folder, method_ids), ".json")
names(method_paths) <- method_ids

# Write all the parameters of each method to file.
# The checking of previous parameters is needed so that we don't constantly
# make the files newer, breaking Snakemake.
# FIXME: How can we clean up out of date parameter files?
for (id in method_ids) {
  new <- method_pars[[id]]
  path <- method_paths[[id]]

  if (file.exists(path)) {
    old <- jsonlite::fromJSON(path)
    # Account for small floating point differences.
    if (!isTRUE(all.equal(old, new, tolerance = 1e-2))) {
      # The file already exists, but we changed it.
      write(jsonlite::toJSON(new, null = "null"), path)
    }
  } else {
    # This file doesn't already exist.
    write(jsonlite::toJSON(new, null = "null"), path)
  }
}
